/****************************************************  Channel Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are specific to individual Signal Stream channels.

	Rev History: 1-10-2014:		Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Channel Functions  *****************************************************/

#include <stdlib.h>

#include "adapter.h"
#include "sigfpga.h"
#include "common_functions.h"
#include "channel_functions.h"
#include "dma_functions.h"

// Set the channel interrupt mask bits to enable (1) or disable (0) interrupt generation for each event type.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ChannelMask(s_Adapter *p_HW, unsigned long ChannelNumber, s_ChannelIntMask ChannelIntMask)
{
	return(Write(p_HW, BAR2, ChannelNumber << 12 | CHANNEL_INT_MASK,
				(unsigned long long)ChannelIntMask.DMAMarker |
				(unsigned long long)ChannelIntMask.Error[0] << 4 |
				(unsigned long long)ChannelIntMask.Error[1] << 5));
}

// Clear the channel status register.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ClearChannelStatus(s_Adapter *p_HW, unsigned long ChannelNumber)
{
	// Clear the channel status register.
	return(Write(p_HW, BAR2, ChannelNumber << 12 | CHANNEL_STATUS, 0x0ull));
}

// Start (OnOff = 1) or stop (OnOff = 0) channel processing.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long FlowControl(s_Adapter *p_HW, unsigned long ChannelNumber, unsigned long OnOff)
{
	return(Write(p_HW, BAR2, ChannelNumber << 12 | SYNCHRONIZER_CTL, (unsigned long long)OnOff));
}

// Report the current on/off status of the requested channel.
// Return of zero indicates the channel is off, one indicates the channel is on.
unsigned long FlowStatus(s_Adapter *p_HW, unsigned long ChannelNumber)
{
	unsigned long long Value = 0;
	
	Read(p_HW, BAR2, ChannelNumber << 12 | SYNCHRONIZER_CTL, &Value);
	
	return((unsigned long)(Value & 0x0000000000000001ull));
}

// Report the channel status and clear the values.
// Returns the total number of status events detected.
unsigned long GetChannelStatus(s_Adapter *p_HW, unsigned long ChannelNumber, s_ChannelStatus *p_ChannelStatus)
{
	unsigned long long Value = 0;
	unsigned long Status = 0;
	size_t cnt = 0;
	
	// Save and clear the contents of the channel status register.
	Write(p_HW, BAR2, ChannelNumber << 12 | CHANNEL_STATUS, 0x0ull);
	// Read the channel status register.
	Read(p_HW, BAR2, ChannelNumber << 12 | CHANNEL_STATUS, &Value);

	// Strip the DMA marker bit.
	if ((p_ChannelStatus->DMAMarker = Value & 0x0000000000000001ull)) Status++;

	// Strip the individual channel error indicator bits.
	for (cnt = 0; cnt < 2; cnt++)
		if ((p_ChannelStatus->Error[cnt] = Value >> ((cnt + 4)) & 0x0000000000000001ull)) Status++;

	// Return the total fault count.
	return(Status);
}

// Set all of the channel configuration registers.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LoadChannel(s_Adapter *p_HW, unsigned long ChannelNumber, s_Channel Channel)
{
	unsigned long Status = 0;
	
		// Allocate DMA buffers for the selected channel.
		Status = 100*DMAAllocate(p_HW, ChannelNumber, Channel.DMABuffer);
		
		// Load the SYNCHRONIZER configuration register for the selected channel.
		Status += LoadSynchronizer(p_HW, ChannelNumber, Channel.Synchronizer);
		
	return(Status);
}

// Configure the synchronizer operating controls.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LoadSynchronizer(s_Adapter *p_HW, unsigned long ChannelNumber, s_Synchronizer Synchronizer)
{
	return(Write(p_HW, BAR2, ChannelNumber << 12 | SYNCHRONIZER_CMD,
			(unsigned long long)Synchronizer.StartEvent |
			(unsigned long long)Synchronizer.StopEvent << 2 |
			(unsigned long long)Synchronizer.SampleCount << 32));
}

// Report the current ready status of the requested channel.
// Return of zero indicates the channel is not ready for a start trigger, one indicates the channel is ready.
unsigned long ReadyStatus(s_Adapter *p_HW, unsigned long ChannelNumber)
{
	unsigned long long Value = 0;
	
	Read(p_HW, BAR2, ChannelNumber << 12 | SYNCHRONIZER_CTL, &Value);
	
	// Shift the ready status bit so that the function returns either one or zero.
	return((unsigned long)((Value >> 1) & 0x0000000000000001ull));
}

// Initialize the structure members assigned to the user defined pointer attached to the hardware handle.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long UserInitialize(s_User *p_User, unsigned long ChannelNumber, s_DMABuffer DMABuffer, unsigned long *p_DataBuffer)
{
	unsigned long Status = 0;
	size_t index = 0;
	
	// Initialize the DMA marker index to the first set of pages.
	p_User->MarkerIndex[ChannelNumber-1] = 0;
	// Compute the maximum marker index based on the DMA buffer settings.
	p_User->MaxMarkerIndex[ChannelNumber-1] = (DMABuffer.PageCount / DMABuffer.PagesPerMark) - 1;
	for (index = 0; index < DMABuffer.PagesPerMark; index++)
	{
		p_User->p_DataBuffer[ChannelNumber-1][index] = malloc(DMABuffer.BurstSizeB * DMABuffer.BurstCount);
		if (p_User->p_DataBuffer[ChannelNumber-1][index] == NULL)
		{
			// Do not continue because there is no place to store data.
			Status++;
			// Mark this data buffer as a failed allocation.
			p_DataBuffer[index] = 1;
		}
		else
			// Mark this data buffer as a successful allocation.
			p_DataBuffer[index] = 0;
	}
	// Reset the error counters.
	for (index = 0; index < 3; index++)
		p_User->Error[ChannelNumber-1][index] = 0;

	return(Status);
}
