/****************************************************  Device Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are used to initialize the hardware specific to each SigFPGA model.

	Rev History: 7-22-2014:		Added Model 377 support.
				 1-10-2014:		Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Device Functions  *****************************************************/

#include <stdlib.h>

#include "adapter.h"
#include "sigfpga.h"
#include "common_functions.h"
#include "channel_functions.h"
#include "device_functions.h"


// Perform a serial read transaction to the AD9512 clock distribution chip.
// Return the value read back from the clock distribution chip.
unsigned long AD9512Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the AD9512 register at the requested address.
	Serial(p_HW, SPI, RISING_EDGE, (0x1 << (AD9512_INST-1)) | Address, AD9512_INST, 0x0, AD9512_DATA, &Value);

	return((unsigned long)Value);
}

// Perform a serial write transaction to the AD9512 clock distribution chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long AD9512Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the ADC12D1600 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, Address, AD9512_INST, Data, AD9512_DATA, &Value);

	return(Status);
}

// Initiate a calibration cycle on the ADC12D1600 ADC.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADC12D1600Cal(s_Adapter *p_HW, unsigned long ChipSelect)
{
	unsigned long Status = 0;

	// Enable calibration by performing a read-modify-write operation to the configuration register (0x0).
	Status += ADC12D1600Write(p_HW, ChipSelect, 0x0, ADC12D1600Read(p_HW, ChipSelect, 0x0) | 0x8000);

	// Wait 50 mseconds for the calibration to complete.
	MicrosecondTimer(50000);

	// Disable calibration by performing a read-modify-write operation to the configuration register (0x0).
	Status += ADC12D1600Write(p_HW, ChipSelect, 0x0, ADC12D1600Read(p_HW, ChipSelect, 0x0) & 0x7FFF);

	return(Status);
}

// Perform a serial read transaction to the ADC12D1600 ADC.
// Return the value read back from the chip.
unsigned long ADC12D1600Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the ADC12D1600 register at the requested address.
	Serial(p_HW, SPI, RISING_EDGE, (0x1 << (ADC12D1600_INST-1)) | 0x40 | (Address << 1), ADC12D1600_INST, 0x0, ADC12D1600_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the ADC12D1600 ADC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADC12D1600Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the ADC12D1600 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, 0x40 | (Address << 1), ADC12D1600_INST, Data, ADC12D1600_DATA, &Value);

	return(Status);
}

// Set the ADL5566 amplifier enables.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADL5566Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long PortNumber, unsigned long Data)
{
	// Performa a read-modify-write operation to the command status port expander output settings register (0x5).
	if (Data)
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) | (0x1 << PortNumber)));
	else
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) & (0xFEFF >> (8-PortNumber))));
}

// Perform a serial read transaction to the ADS42LB69 ADC chip.
// Return the value read back from the clock distribution chip.
unsigned long ADS42LB69Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the AD9512 register at the requested address.
	Serial(p_HW, SPI, RISING_EDGE, (0x1 << (ADS42LB69_INST-1)) | Address, ADS42LB69_INST, 0x0, ADS42LB69_DATA, &Value);

	return((unsigned long)Value);
}

// Perform a serial write transaction to the ADS42LB69 ADC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADS42LB69Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the ADC12D1600 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, Address, ADS42LB69_INST, Data, ADS42LB69_DATA, &Value);

	return(Status);
}

// Perform a serial read transaction to the FXL6408 command/status port expander chip.
// Return the value read back from the chip.
unsigned long FXL6408Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Read the FXL6408 register at the requested address.
	Serial(p_HW, I2C, I2C_READ, (ChipSelect << 16) | (Address << 8) | ChipSelect | 0x1, 0x18, 0x0, FXL6408_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the FXL6408 command/status port expander chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long FXL6408Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Value = 0;

	// Write data to the FXL6408 register at the requested address.
	return(Serial(p_HW, I2C, I2C_WRITE, (ChipSelect << 8) | Address, 0x10, Data, FXL6408_DATA, &Value));
}

// Perform a serial read transaction to the INA3221 voltage monitor.
// Return the value read back from the chip.
unsigned long INA3221Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Read the FXL6408 register at the requested address.
	Serial(p_HW, I2C, I2C_READ, (ChipSelect << 16) | (Address << 8) | ChipSelect | 0x1, 0x18, 0x0, INA3221_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the INA3221 voltage monitor.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long INA3221Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Value = 0;

	// Write data to the FXL6408 register at the requested address.
	return(Serial(p_HW, I2C, I2C_WRITE, (ChipSelect << 8) | Address, 0x10, Data, INA3221_DATA, &Value));
}

// Perform a serial write transaction to the LTC1661 DAC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LTC1661Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the LTC1661 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, Address, LTC1661_INST, Data << 2, LTC1661_DATA, &Value);

	return(Status);
}

// Perform a serial read transaction to the LTC2991 voltage/current/temperature monitor.
// Return the value read back from the chip.
unsigned long LTC2991Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Read the FXL6408 register at the requested address.
	Serial(p_HW, I2C, I2C_READ, (ChipSelect << 16) | (Address << 8) | ChipSelect | 0x1, 0x18, 0x0, LTC2991_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the LTC2991 voltage/current/temperature monitor.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LTC2991Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Value = 0;

	// Write data to the FXL6408 register at the requested address.
	return(Serial(p_HW, I2C, I2C_WRITE, (ChipSelect << 8) | Address, 0x10, Data, LTC2991_DATA, &Value));
}

// Intitialize the Model 376 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M376Initialize(s_Adapter *p_HW, s_M376Devices M376Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The standard Model 376 has two receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The SigFPGA hardware sign extends the 12-bit sample data to a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0x38);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0xC7);

	// Configure the ADC from the default structure values.
	for (cnt = 0; cnt < 11; cnt++)
		Status += ADC12D1600Write(p_HW, M376_RX1TO2_ADC, M376Devices.RX1TO2_ADC.Payload[cnt][0], M376Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M376_RX1_TRIM_DAC, M376Devices.RX1_TRIM_DAC.Payload[cnt][0], M376Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M376_RX2_TRIM_DAC, M376Devices.RX2_TRIM_DAC.Payload[cnt][0], M376Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
		Status += ADL5566Write(p_HW, CMD_STAT1, M376Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M376Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);

	return(Status);
}

// Intitialize the Model 377 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M377Initialize(s_Adapter *p_HW, s_M377Devices M377Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The Model 377 can be equipped with either two or four receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 377 RX channels produce a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Configure the first ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M377_RX1TO2_ADC, M377Devices.RX1TO2_ADC.Payload[cnt][0], M377Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the second ADC that is assigned to RX channels #3 and #4 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M377_RX3TO4_ADC, M377Devices.RX3TO4_ADC.Payload[cnt][0], M377Devices.RX3TO4_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M377_RX1_TRIM_DAC, M377Devices.RX1_TRIM_DAC.Payload[cnt][0], M377Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX2_TRIM_DAC, M377Devices.RX2_TRIM_DAC.Payload[cnt][0], M377Devices.RX2_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX3_TRIM_DAC, M377Devices.RX3_TRIM_DAC.Payload[cnt][0], M377Devices.RX3_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX4_TRIM_DAC, M377Devices.RX4_TRIM_DAC.Payload[cnt][0], M377Devices.RX4_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
	{
//		Status += ADL5566Write(p_HW, CMD_STAT1, M377Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M377Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);
//		Status += ADL5566Write(p_HW, CMD_STAT1, M377Devices.RX3TO4_AMP_ENBL.Enable[cnt][0], M377Devices.RX3TO4_AMP_ENBL.Enable[cnt][1]);
	}

	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the selected input to the clock distribution chip.
		// Note: The default input is the on-board synthesizer.
		M377Devices.ClockDist.Payload[9][1] = 0x02;

	// Hardware reset the clock distribution chip.
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x0000, 0x30);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x005A, 0x01);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x0000, 0x10);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x005A, 0x01);
	// Configure the clock distribution chip by loading the default values.
	for (cnt = 0; cnt < 22; cnt++)
		Status += AD9512Write(p_HW, M377_CLK_DIST, M377Devices.ClockDist.Payload[cnt][0], M377Devices.ClockDist.Payload[cnt][1]);

	return(Status);
}
